# -*- coding: utf-8 -*-
"""
    # Clay.pp_pygments

    Highlight codeblocks with Pygments by passing HTML through it.

"""
import re

try:
    import pygments
    from pygments import highlight
    from pygments.formatters import HtmlFormatter
    from pygments.lexers import get_lexer_by_name, ClassNotFound
    enabled = True
except ImportError:
    enabled = False


RX_CODEBLOCK = re.compile(r'<pre(?: lang="([a-z0-9]+#?)")?><code'
    '(?: class="([a-z0-9_\-]+#?).*?")?>(.*?)</code></pre>',
    re.IGNORECASE | re.DOTALL)


def _unescape_html(html):
    html = html.replace('&lt;', '<')
    html = html.replace('&gt;', '>')
    html = html.replace('&amp;', '&')
    html = html.replace('&quot;', '"')
    return html


def _highlight_match(match):
    language, classname, code = match.groups()
    lang_or_class = language or classname

    if lang_or_class is None:
        return match.group(0)

    linenos = False
    if lang_or_class.endswith('#'):
        lang_or_class = lang_or_class[:-1]
        linenos = True

    try:
        lexer = get_lexer_by_name(lang_or_class)
    except ClassNotFound:
        return match.group(0)
    
    formatter = HtmlFormatter(
        cssclass='highlight ' + lang_or_class,
        linenos=linenos
    )

    return highlight(_unescape_html(code), lexer, formatter)


def process(html):
    return RX_CODEBLOCK.sub(_highlight_match, html)




#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
# Clay.manage

Command line scripts.

"""
import io
import os
import sys

from shake import manager, json
import voodoo
import yaml

from .config import SOURCE_DIR
from .core import Clay


WELCOME = """
\033[1m Clay \033[0m - A rapid prototyping tool by Lucuma labs"""

SKELETON = os.path.join(os.path.dirname(os.path.realpath(__file__)),
    'skeleton')

SKELETON_HELP = """
    Done!
    Now go to %(new_app_path)s, and run `clay run` to start the server.
    Don't forget to read the README.md
    """

SOURCE_NOT_FOUND_HELP = """We couldn't found a source dir ('%s' or 'views').
Are you sure you're in the correct folder?
""" % SOURCE_DIR


class SourceDirNotFound(Exception):
    pass


@manager.command
def new(new_app_path='.'):
    """DIR_PATH

    Creates a new project
    """
    new_app_path = new_app_path.rstrip(os.path.sep)
    voodoo.reanimate_skeleton(SKELETON, new_app_path)

    print SKELETON_HELP % {'new_app_path': new_app_path}


@manager.command
def build(theme_prefix=''):
    """.

    Generates a static version of the site
    """
    try:
        proto = get_current(theme_prefix)
    except SourceDirNotFound:
        print SOURCE_NOT_FOUND_HELP
        return
    proto.build()


@manager.command
def run(theme_prefix=''):
    """.

    Run the development server
    """
    try:
        proto = get_current(theme_prefix)
    except SourceDirNotFound:
        print SOURCE_NOT_FOUND_HELP
        return
    proto.run()


@manager.command
def version():
    """.

    Prints the current Clay version
    """
    import clay
    print clay.__version__

#------------------------------------------------------------------------------

def get_current(theme_prefix='', cwd=None):
    cwd = '.' if cwd is None else cwd
    cwd = os.path.abspath(cwd)
    # print cwd
    source_dir = get_source_dir(cwd)
    settings = get_settings(cwd)
    settings.setdefault('theme_prefix', theme_prefix)
    
    return Clay(cwd, settings, source_dir=source_dir)


def get_source_dir(cwd):
    sources = [SOURCE_DIR, 'views']
    for source in sources:
        if os.path.exists(os.path.join(cwd, source)):
            return source
    raise SourceDirNotFound


def get_settings(cwd, filename='settings.yml'):
    settings = {}
    settings_filepath = os.path.join(cwd, filename)
    if os.path.isfile(settings_filepath):
        with io.open(settings_filepath) as f:
            source = f.read()
        settings = yaml.load(source)

    return settings


def main():
    manager.run()
    print '\n'


if __name__ == "__main__":
    main()




# -*- coding: utf-8 -*-
import io
import os

from clay import Clay


HTTP_OK = 200
HTTP_NOT_FOUND = 404

clay_ = Clay(__file__)
c = clay_.test_client()


def get_views_filepath(filename):
    return os.path.join(clay_.source_dir, filename)


def get_build_filepath(filename):
    return os.path.join(clay_.build_dir, filename)


def make_file(filepath, content):
    if not isinstance(content, unicode):
        content = unicode(content, 'utf-8')
    with io.open(filepath, 'w+t') as f:
        f.write(content)
    return filepath


def make_view(filename, content):
    filepath = get_views_filepath(filename)
    make_file(filepath, content)
    return filepath


def read_file(filepath):
    with io.open(filepath) as f:
        content = f.read()
    content = content.encode('utf8')
    return content


def remove_file(filepath):
    try:
        os.remove(filepath)
    except OSError:
        pass




    # -*- coding: utf-8 -*-
"""
    # Clay.p_markdown

    Text-to-HTML conversion tool for web writers

    http://daringfireball.net/projects/markdown/
    http://packages.python.org/Markdown

"""
import os
import re

from jinja2.ext import Extension
try:
    import markdown
    enabled = True
except ImportError:
    enabled = False

from .utils import to_unicode


extensions_in = ('.md', '.markdown', '.mdown', '.html.md', '.html.mdown', '.html.markdown',)
extension_out = '.html'


def get_metadata(md):
    meta = md.Meta
    # Flatten single items lists
    for k, v in meta.items():
        if len(v) == 1:
            meta[k] = v[0]
    return meta


def find_first_title(source):
    first_title = ''
    m = re.search(RX_FIRST_TITLE, source)
    if m:
        first_title = m.group(1)
    return first_title


def add_extensions(clay):
    from .libs.md_fenced_gh import FencedCodeGhExtension
    from .libs.md_superscript import SuperscriptExtension
    from .libs.md_toc import TocExtension

    RX_FIRST_TITLE = re.compile(r'^\s*#\s*(?P<value>.*)\n')

    md_options = {
        'extensions': [
            'meta',
            'abbr',
            'def_list',
            'footnotes',
            'nl2br',
            'smart_strong',
            'tables',
            'attr_list',
            FencedCodeGhExtension(),
            SuperscriptExtension(),
            TocExtension(),
        ],
        'output_format': 'html5',
        'safe_mode': False,
        'tab_length': 4,
        'enable_attributes': True,
        'smart_emphasis': True,
        'lazy_ol': True
    }
    md = markdown.Markdown(**md_options)
    md.preprocessors['html_block'].markdown_in_raw = True
    theme_prefix = clay.settings.get('theme_prefix', '')


    class MarkdownExtension(Extension):

        def preprocess(self, source, name, filename=None):
            if name is None or os.path.splitext(name)[1] not in extensions_in:
                return source

            source = to_unicode(source)
            html = md.convert(source)
            metadata = get_metadata(md)
            toc_page = md.toc
            md.reset()

            template = metadata.pop('template', None)

            if template:
                # Using this you can have multiple themes (HTML, epub, etc.)!
                template = theme_prefix + template

                page_title = metadata.pop('title', None)
                if not page_title:
                    page_title = find_first_title(source)

                content = [
                    '{% extends "', template, '" %}\n',
                    '{% block title %}', page_title, '{% endblock %}\n',
                ]
                content.extend([
                    '{% block ' + key + ' %}' + value + '{% endblock %}\n'
                    for key, value in metadata.items()
                ])

                content.extend([
                    '{% block content %}', html, '{% endblock %}\n',
                    '{% block toc %}', toc_page, '{% endblock %}\n',
                ])
            else:
                content = [
                    '{% set ' + key + " = '''" + value + "''' %}\n"
                    for key, value in metadata.items()
                ]
                content.append(html)
            
            return ''.join(content)

    clay.render.add_extension(MarkdownExtension)




# -*- coding: utf-8 -*-
"""
    # Typographer

    Typogrify without dependencies

"""
import re


# Some of the filters will ignore the text inside this tags
RX_TAGS_TO_SKIP = re.compile("<(/)?(?:pre|code|kbd|script|math|svg)[^>]*>",
    re.IGNORECASE)


def typogrify(text):
    """The super typography filter
    
    Applies the following filters: wrap_amps, widont, educate_dashes, 
    educate_ellipses, educate_quotes, wrap_caps, and initial_quotes.
    
    >>> typogrify('<h2>"Valhallen" & MM fans act obnoxiously</h2>')
    u'<h2><span class="dquo">&#8220;</span>Valhallen&#8221; <span class="amp">&amp;</span> <span class="caps">MM</span> fans act&nbsp;obnoxiously</h2>'

    """
    text = _to_unicode(text)
    text = wrap_amps(text)
    text = widont(text)

    # escapes, dashes, ellipses, quotes and caps
    text = dexter_lab(text)

    text = initial_quotes(text)
    return text


def dexter_lab(text, escapes=True, dashes=True, ellipses=True, 
        quotes=True, caps=True, DeeDee=False):
    """
    - Straight quotes ( " and ' ) into "curly" quote HTML entities.
    - Dashes (``--`` and ``---``) into em- and en-dash entities.
    - Three consecutive dots (``...`` or ``. . .``) into an ellipsis entity.
    - Wraps multiple capital letters in ``<span class="caps">`` 
    so they can be styled with CSS. 

    """
    text = _to_unicode(text)
    tokens = _tokenize(text)
    result = []
    in_skipped_tag = False 
    prev_token_last_char = ""
    
    for token in tokens:
        if token[0] == "tag":
            # Don't mess with tags.
            result.append(token[1])
            close_match = RX_TAGS_TO_SKIP.match(token[1])
            if close_match and close_match.group(1) == None:
                in_skipped_tag = True
            else:
                in_skipped_tag = False
        else:
            t1 = token[1]
            last_char = t1[-1:]

            if not in_skipped_tag:
                if escapes:
                    t1 = process_escapes(t1)
                if dashes:
                    t1 = educate_dashes(t1)
                if ellipses:
                    t1 = educate_ellipses(t1)
                if quotes:
                    t1 = re.sub('&quot;', '"', t1)
                    if t1 == "'":
                        # Special case: single-character ' token
                        if re.match("\S", prev_token_last_char):
                            t1 = "&#8217;"
                        else:
                            t1 = "&#8216;"
                    elif t1 == '"':
                        # Special case: single-character " token
                        if re.match("\S", prev_token_last_char):
                            t1 = "&#8221;"
                        else:
                            t1 = "&#8220;"
                    else:
                        # Normal case:
                        t1 = educate_quotes(t1)
                if caps:
                    t1 = _cap_finder.sub(_cap_wrapper, t1)

            prev_token_last_char = last_char
            result.append(t1)

    output = "".join(result)
    return output


_cap_finder = re.compile(r"""(
    # Group 2: Any amount of caps and digits
    (\b[A-Z\d]*
    # A cap string much at least include two caps (but they can have digits between them)
    [A-Z]\d*[A-Z]
    # Any amount of caps and digits or dumb apostsrophes
    [A-Z\d']*\b)
    # OR: Group 3: Some caps, followed by a '.' and an optional space
    | (\b[A-Z]+\.\s?
    # Followed by the same thing at least once more
    (?:[A-Z]+\.\s?)+)
    (?:\s|\b|$))
    """, re.VERBOSE)


def _cap_wrapper(matchobj):
    """This is necessary to keep dotted cap strings to pick up extra
    spaces."""
    if matchobj.group(2):
        return """<span class="caps">%s</span>""" % matchobj.group(2)
    else:
        if matchobj.group(3)[-1] == " ":
            caps = matchobj.group(3)[:-1]
            tail = ' '
        else:
            caps = matchobj.group(3)
            tail = ''
        return """<span class="caps">%s</span>%s""" % (caps, tail)


def educate_dashes(text):
    """Returns the text, with each instance of "--" translated to
    an em-dash HTML entity.

    >>> educate_dashes('The years 2001---2003')
    u'The years 2001&#8212;2003'
    >>> educate_dashes('You are the one--the only one--who offered help')
    u'You are the one&#8211;the only one&#8211;who offered help'

    """
    text = _to_unicode(text)
    text = re.sub(r"""---""", r"""&#8212;""", text) # en  (yes, backwards)
    text = re.sub(r"""--""", r"""&#8211;""", text) # em (yes, backwards)
    return text


def educate_ellipses(text):
    """Returns the text, with each instance of "..." translated to
    an ellipsis HTML entity.
    
    >>> educate_ellipses('Huh...?')
    u'Huh&#8230;?'

    """
    text = _to_unicode(text)
    text = re.sub(r"""\.\.\.""", r"""&#8230;""", text)
    text = re.sub(r"""\. \. \.""", r"""&#8230;""", text)
    return text


def educate_quotes(text):
    r"""Returns the text, with "educated" curly quote HTML entities.
    
    >>> educate_quotes('"Isn\'t this fun?"')
    u'&#8220;Isn&#8217;t this fun?&#8221;'

    """
    text = _to_unicode(text)
    punct_class = r"""[!"#\$\%'()*+,-.\/:;<=>?\@\[\\\]\^_`{|}~]"""

    # Special case if the very first character is a quote
    # followed by punctuation at a non-word-break. Close the quotes by brute force:
    text = re.sub(r"""^'(?=%s\\B)""" % (punct_class,), r"""&#8217;""", text)
    text = re.sub(r"""^"(?=%s\\B)""" % (punct_class,), r"""&#8221;""", text)

    # Special case for double sets of quotes, e.g.:
    #   <p>He said, "'Quoted' words in a larger quote."</p>
    text = re.sub(r""""'(?=\w)""", """&#8220;&#8216;""", text)
    text = re.sub(r"""'"(?=\w)""", """&#8216;&#8220;""", text)

    # Special case for decade abbreviations (the '80s):
    text = re.sub(r"""\b'(?=\d{2}s)""", r"""&#8217;""", text)

    close_class = r"""[^\ \t\r\n\[\{\(\-]"""
    dec_dashes = r"""&#8211;|&#8212;"""

    # Get most opening single quotes:
    opening_single_quotes_regex = re.compile(r"""
        (
            \s          |   # a whitespace char, or
            &nbsp;      |   # a non-breaking space entity, or
            --          |   # dashes, or
            &[mn]dash;  |   # named dash entities
            %s          |   # or decimal entities
            &\#x201[34];    # or hex
        )
        '                 # the quote
        (?=\w)            # followed by a word character
        """ % (dec_dashes,), re.VERBOSE)

    text = opening_single_quotes_regex.sub(r"""\1&#8216;""", text)

    closing_single_quotes_regex = re.compile(r"""
        (%s)
        '
        (?!\s | s\b | \d)
        """ % (close_class,), re.VERBOSE)

    text = closing_single_quotes_regex.sub(r"""\1&#8217;""", text)

    closing_single_quotes_regex = re.compile(r"""
        (%s)
        '
        (\s | s\b)
        """ % (close_class,), re.VERBOSE)

    text = closing_single_quotes_regex.sub(r"""\1&#8217;\2""", text)

    # Any remaining single quotes should be opening ones:
    text = re.sub(r"""'""", r"""&#8216;""", text)

    # Get most opening double quotes:
    opening_double_quotes_regex = re.compile(r"""
        (
            \s          |   # a whitespace char, or
            &nbsp;      |   # a non-breaking space entity, or
            --          |   # dashes, or
            &[mn]dash;  |   # named dash entities
            %s          |   # or decimal entities
            &\#x201[34];    # or hex
        )
        "                 # the quote
        (?=\w)            # followed by a word character
        """ % (dec_dashes,), re.VERBOSE)

    text = opening_double_quotes_regex.sub(r"""\1&#8220;""", text)

    # Double closing quotes:
    closing_double_quotes_regex = re.compile(r"""
        #(%s)?   # character that indicates the quote should be closing
        "
        (?=\s)
        """ % (close_class,), re.VERBOSE)

    text = closing_double_quotes_regex.sub(r"""&#8221;""", text)

    closing_double_quotes_regex = re.compile(r"""
        (%s)   # character that indicates the quote should be closing
        "
        """ % (close_class,), re.VERBOSE)

    text = closing_double_quotes_regex.sub(r"""\1&#8221;""", text)

    # Any remaining quotes should be opening ones.
    text = re.sub(r'"', r"""&#8220;""", text)

    return text


def initial_quotes(text):
    """Wraps initial quotes in ``class="dquo"`` for double quotes or  
    ``class="quo"`` for single quotes. Works in these block tags
    ``(h1-h6, p, li, dt, dd)`` and also accounts for potential opening inline
    elements ``a, em, strong, span, b, i``
    
    >>> initial_quotes('"With primes"')
    u'<span class="dquo">"</span>With primes"'
    >>> initial_quotes("'With single primes'")
    u'<span class="quo">\\'</span>With single primes\\''
    
    >>> initial_quotes('<a href="#">"With primes and a link"</a>')
    u'<a href="#"><span class="dquo">"</span>With primes and a link"</a>'
    
    >>> initial_quotes('&#8220;With smartypanted quotes&#8221;')
    u'<span class="dquo">&#8220;</span>With smartypanted quotes&#8221;'
    """
    text = _to_unicode(text)
    quote_finder = re.compile(r"""
        # start with an opening p, h1-6, li, dd, dt or the start of the string
        ((<(p|h[1-6]|li|dt|dd)[^>]*>|^)
        \s*
        # optional opening inline tags, with more optional white space for each.
        (<(a|em|span|strong|i|b)[^>]*>\s*)*)  
        # Find me a quote! (only need to find the left quotes and the primes)
        # double quotes are in group 7, singles in group 8 
        (("|&ldquo;|&\#8220;)|('|&lsquo;|&\#8216;))  
        """, re.VERBOSE)

    def _quote_wrapper(matchobj):
        if matchobj.group(7): 
            classname = "dquo"
            quote = matchobj.group(7)
        else:
            classname = "quo"
            quote = matchobj.group(8)
        return """%s<span class="%s">%s</span>""" % (
            matchobj.group(1), classname, quote)

    output = quote_finder.sub(_quote_wrapper, text)
    return output


def process_escapes(text):
    r"""Returns the string, with after processing the following backslash
    escape sequences. This is useful if you want to force a "dumb"
    quote or other character to appear.

        Escape  Value
        ------  -----
        \\      &#92;
        \"      &#34;
        \'      &#39;
        \.      &#46;
        \-      &#45;
        \`      &#96;
    """
    text = re.sub(r"""\\\\""", r"""&#92;""", text)
    text = re.sub(r'''\\"''', r"""&#34;""", text)
    text = re.sub(r"""\\'""", r"""&#39;""", text)
    text = re.sub(r"""\\\.""", r"""&#46;""", text)
    text = re.sub(r"""\\-""", r"""&#45;""", text)
    text = re.sub(r"""\\`""", r"""&#96;""", text)

    return text


def widont(text):
    """Replaces the space between the last two words in a string with ``&nbsp;``
    Works in these block tags ``(h1-h6, p, li, dd, dt)`` and also accounts for 
    potential closing inline elements ``a, em, strong, span, b, i``
    
    >>> widont('A very simple test')
    u'A very simple&nbsp;test'

    Single word items shouldn't be changed
    >>> widont('Test')
    u'Test'
    >>> widont(' Test')
    u' Test'
    >>> widont('<ul><li>Test</p></li><ul>')
    u'<ul><li>Test</p></li><ul>'
    >>> widont('<ul><li> Test</p></li><ul>')
    u'<ul><li> Test</p></li><ul>'
    
    >>> widont('<p>In a couple of paragraphs</p><p>paragraph two</p>')
    u'<p>In a couple of&nbsp;paragraphs</p><p>paragraph&nbsp;two</p>'
    
    >>> widont('<h1><a href="#">In a link inside a heading</i> </a></h1>')
    u'<h1><a href="#">In a link inside a&nbsp;heading</i> </a></h1>'
    
    >>> widont('<h1><a href="#">In a link</a> followed by other text</h1>')
    u'<h1><a href="#">In a link</a> followed by other&nbsp;text</h1>'

    Empty HTMLs shouldn't error
    >>> widont('<h1><a href="#"></a></h1>') 
    u'<h1><a href="#"></a></h1>'
    
    >>> widont('<div>Divs get no love!</div>')
    u'<div>Divs get no love!</div>'
    
    >>> widont('<pre>Neither do PREs</pre>')
    u'<pre>Neither do PREs</pre>'
    
    >>> widont('<div><p>But divs with paragraphs do!</p></div>')
    u'<div><p>But divs with paragraphs&nbsp;do!</p></div>'
    """
    text = _to_unicode(text)
    widont_finder = re.compile(r"""
        # must be proceeded by an approved inline opening or closing tag or
        # a nontag/nonspace
        ((?:</?(?:a|em|span|strong|i|b)[^>]*>)|[^<>\s])
        # the space to replace
        \s+
        # must be followed by non-tag non-space characters                                       
        ([^<>\s]+
        \s*
        # optional closing inline tags with optional white space after each
        (</(a|em|span|strong|i|b)>\s*)*
        # end with a closing p, h1-6, li or the end of the string
        ((</(p|h[1-6]|li|dt|dd)>)|$))
        """, re.VERBOSE)
    output = widont_finder.sub(r'\1&nbsp;\2', text)
    return output


def wrap_amps(text):
    """Wraps apersands in HTML with ``<span class="amp">`` so they can be
    styled with CSS. Apersands are also normalized to ``&amp;``. Requires 
    ampersands to have whitespace or an ``&nbsp;`` on both sides.
    
    >>> wrap_amps('One & two')
    u'One <span class="amp">&amp;</span> two'
    >>> wrap_amps('One &amp; two')
    u'One <span class="amp">&amp;</span> two'
    >>> wrap_amps('One &#38; two')
    u'One <span class="amp">&amp;</span> two'

    >>> wrap_amps('One&nbsp;&amp;&nbsp;two')
    u'One&nbsp;<span class="amp">&amp;</span>&nbsp;two'

    It won't mess up & that are already wrapped, in entities or URLs

    >>> wrap_amps('One <span class="amp">&amp;</span> two')
    u'One <span class="amp">&amp;</span> two'
    >>> wrap_amps('&ldquo;this&rdquo; & <a href="/?that&amp;test">that</a>')
    u'&ldquo;this&rdquo; <span class="amp">&amp;</span> <a href="/?that&amp;test">that</a>'

    It should ignore standalone amps that are in attributes
    >>> wrap_amps('<link href="xyz.html" title="One & Two">xyz</link>')
    u'<link href="xyz.html" title="One & Two">xyz</link>'

    """
    text = _to_unicode(text)
    # tag_pattern from
    # http://haacked.com/archive/2004/10/25/usingregularexpressionstomatchhtml.aspx
    # it kinda sucks but it fixes the standalone amps in attributes bug
    tag_pattern = '</?\w+((\s+\w+(\s*=\s*(?:".*?"|\'.*?\'|[^\'">\s]+))?)+\s*|\s*)/?>'
    amp_finder = re.compile(r"(\s|&nbsp;)(&|&amp;|&\#38;)(\s|&nbsp;)")
    intra_tag_finder = re.compile(r'(?P<prefix>(%s)?)(?P<text>([^<]*))(?P<suffix>(%s)?)' % (tag_pattern, tag_pattern))

    def _amp_process(groups):
        prefix = groups.group('prefix') or ''
        text = amp_finder.sub(r"""\1<span class="amp">&amp;</span>\3""", groups.group('text'))
        suffix = groups.group('suffix') or ''
        return prefix + text + suffix

    output = intra_tag_finder.sub(_amp_process, text)
    return output


def wrap_caps(text):
    r"""Wraps multiple capital letters in ``<span class="caps">`` 
    so they can be styled with CSS. 
    
    >>> wrap_caps("A message from KU")
    u'A message from <span class="caps">KU</span>'
    
    >>> wrap_caps("<PRE>CAPS</pre> more CAPS")
    u'<PRE>CAPS</pre> more <span class="caps">CAPS</span>'

    >>> wrap_caps("A message from 2KU2 with digits")
    u'A message from <span class="caps">2KU2</span> with digits'
    
    Dotted caps followed by spaces should never include them in the wrap
    >>> wrap_caps("wrap D.O.T.   like so.")
    u'wrap <span class="caps">D.O.T.</span>  like so.'

    All caps with with apostrophes in them shouldn't break. Only handles
    dump apostrophes though.
    >>> wrap_caps("JIMMY'S")
    u'<span class="caps">JIMMY\'S</span>'

    >>> wrap_caps("<i>D.O.T.</i>HE34T<b>RFID</b>")
    u'<i><span class="caps">D.O.T.</span></i><span class="caps">HE34T</span><b><span class="caps">RFID</span></b>'
    """
    text = _to_unicode(text)
    return dexter_lab(text, escapes=False, dashes=False, ellipses=False, 
        quotes=False, caps=True)


# UTILS ---------------------------------------------------------------------

def _is_protected_type(obj):
    """Determine if the object instance is of a protected type.

    Objects of protected types are preserved as-is when passed to
    to_unicode(strings_only=True).
    """
    return isinstance(obj, (
        types.NoneType,
        int, long,
        datetime.datetime, datetime.date, datetime.time,
        float, Decimal)
    )


def _to_unicode(s, encoding='utf-8', strings_only=False, errors='strict'):
    """Returns a unicode object representing 's'. Treats bytestrings using the
    `encoding` codec.

    If strings_only is True, don't convert (some) non-string-like objects.

    --------------------------------
    Copied almost unchanged from Django <https://www.djangoproject.com/>
    Copyright © 2005-2011 Django Software Foundation.
    Used under the modified BSD license.
    """
    # Handle the common case first, saves 30-40% in performance when s
    # is an instance of unicode.
    if isinstance(s, unicode):
        return s
    if strings_only and _is_protected_type(s):
        return s
    encoding = encoding or 'utf-8'
    try:
        if not isinstance(s, basestring):
            if hasattr(s, '__unicode__'):
                s = unicode(s)
            else:
                try:
                    s = unicode(str(s), encoding, errors)
                except UnicodeEncodeError:
                    if not isinstance(s, Exception):
                        raise
                    # If we get to here, the caller has passed in an Exception
                    # subclass populated with non-ASCII data without special
                    # handling to display as a string. We need to handle this
                    # without raising a further exception. We do an
                    # approximation to what the Exception's standard str()
                    # output should be.
                    s = u' '.join([to_unicode(arg, encoding, strings_only,
                        errors) for arg in s])
        elif not isinstance(s, unicode):
            # Note: We use .decode() here, instead of unicode(s, encoding,
            # errors), so that if s is a SafeString, it ends up being a
            # SafeUnicode at the end.
            s = s.decode(encoding, errors)
    except UnicodeDecodeError, e:
        if not isinstance(s, Exception):
            raise UnicodeDecodeError(s, *e.args)
        else:
            # If we get to here, the caller has passed in an Exception
            # subclass populated with non-ASCII bytestring data without a
            # working unicode method. Try to handle this without raising a
            # further exception by individually forcing the exception args
            # to unicode.
            s = u' '.join([to_unicode(arg, encoding, strings_only,
                errors) for arg in s])
    return s


def _tokenize(text):
    """Takes a text containing HTML markup and returns a reference to an 
    array of the tokens comprising the input string.

    Each token is either a tag (possibly with nested, tags contained therein,
    such as <a href="<MTFoo>">, or a run of text between tags.

    Each element of the array is a two-element array; the first is either 
    'tag' or 'text'; the second is the actual value.
    
    Based on the _tokenize() subroutine from Brad Choate's MTRegex plugin.
        <http://www.bradchoate.com/past/mtregex.php>
    """
    pos = 0
    length = len(text)
    tokens = []

    depth = 6
    nested_tags = "|".join(['(?:<(?:[^<>]',] * depth) + (')*>)' * depth)
    #match = r"""(?: <! ( -- .*? -- \s* )+ > ) |  # comments
    #       (?: <\? .*? \?> ) |  # directives
    #       %s  # nested tags       """ % (nested_tags,)
    tag_soup = re.compile(r"""([^<]*)(<[^>]*>)""")

    token_match = tag_soup.search(text)

    previous_end = 0
    while token_match is not None:
        if token_match.group(1):
            tokens.append(['text', token_match.group(1)])

        tokens.append(['tag', token_match.group(2)])

        previous_end = token_match.end()
        token_match = tag_soup.search(text, token_match.end())

    if previous_end < len(text):
        tokens.append(['text', text[previous_end:]])

    return tokens


def _test():
    import doctest
    doctest.testmod()


if __name__ == "__main__":
    _test()




# -*- coding: utf-8 -*-
"""
    # Clay.pp_typogrify

    Applies the following filters: widont, dashes, ellipses, quotes, caps,
    amp and initial_quotes.

"""
from .libs.typographer import typogrify


enabled = True

def process(html):
    return typogrify(html)



# -*- coding: utf-8 -*-
import shutil

from clay import p_coffee
import pytest

from tests.utils import *


def setup_module():
    try:
        shutil.rmtree(clay_.build_dir)
    except OSError:
        pass


def teardown_module():
    try:
        shutil.rmtree(clay_.build_dir)
    except OSError:
        pass


SRC_COFFEESCRIPT = """
square = (x) -> x * x
cube   = (x) -> square(x) * x
""".strip()


EXPECTED_COFFEESCRIPT = """
(function() {
  var cube, square;

  square = function(x) {
    return x * x;
  };

  cube = function(x) {
    return square(x) * x;
  };

}).call(this);
""".strip()


FILENAME_IN = 'coffee-script.coffee'
FILENAME_OUT = 'coffee-script.js'


SRC_HTML = """
<script src="foo/bar/%s"></script>
<p class="coffee"></p>""" % FILENAME_IN


EXPECTED_HTML = """
<script src="foo/bar/%s"></script>
<p class="coffee"></p>""" % FILENAME_OUT


def test_coffeescript_enabled():
    assert p_coffee.enabled


def test_coffeescript_render():
    filepath = make_view(FILENAME_IN, SRC_COFFEESCRIPT)

    resp = c.get('/' + FILENAME_IN)
    content = resp.data.strip()
    assert content == EXPECTED_COFFEESCRIPT

    remove_file(filepath)


def test_coffeescript_make():
    filepath = make_view(FILENAME_IN, SRC_COFFEESCRIPT)
    filepath_out = get_build_filepath(FILENAME_OUT)
    try:
        clay_.build()
        content = read_file(filepath_out).strip()
        assert content == EXPECTED_COFFEESCRIPT
    finally:
        remove_file(filepath)
        remove_file(filepath_out)


def test_coffeescript_html_replace():
    static_filepath = make_view(FILENAME_IN, SRC_COFFEESCRIPT)
    filepath_out = get_build_filepath(FILENAME_OUT)
    html_filename = 'test_coffee.html'
    html_filepath = make_view(html_filename, SRC_HTML)
    html_filepath_out = get_build_filepath(html_filename)
    try:
        clay_.build()
        content = read_file(html_filepath_out)
        print content
        assert content == EXPECTED_HTML
    finally:
        remove_file(static_filepath)
        remove_file(filepath_out)
        remove_file(html_filepath)
        remove_file(html_filepath_out)




# -*- coding: utf-8 -*-
"""
# Clay.config

"""
import os

SOURCE_DIR = 'source'
BUILD_DIR = 'build'
VIEWS_INDEX = '_index.html'

DEFAULT_TEMPLATES = os.path.join(os.path.dirname(__file__), 'source')

IGNORE = ('.', '_')

default_settings = {
    'host': '0.0.0.0',
    'port': 8080,

    'plain_text': ['.js', ],
    'views_list_ignore': [],

    'pre_processors': ['scss', 'less', 'coffee', 'markdown'],
    'post_processors': ['pygments', 'typogrify'],

    'theme_prefix': '',
}

app_settings = {
    'RELOAD': False,
}




# -*- coding: utf-8 -*-
"""
    # Clay.p_less

    http://lesscss.org/

"""
import os
import subprocess

from jinja2.ext import Extension


COMMAND = 'lessc'

try:
    subprocess.check_output([COMMAND, '--version'])
    enabled = True
except Exception:
    enabled = False

extensions_in = ('.css.less', '.less',)
extension_out = '.css'


def add_extensions(clay):

    class LessExtension(Extension):

        def preprocess(self, source, name, filename=None):
            if name is None or os.path.splitext(name)[1] not in extensions_in:
                return source
            path = os.path.join(clay.source_dir, name)
            try:
                content = subprocess.check_output([COMMAND, path])
            except subprocess.CalledProcessError:
                # Import error?
                content = ''
            return content

    clay.render.add_extension(LessExtension)




# -*- coding: utf-8 -*-
"""
# Clay.utils

"""
from datetime import datetime
import errno
import io
import math
import os
import re
import shutil
try:
    import simplejson as json
except ImportError:
    try:
        import json
    except ImportError:
        pass


RX_ASB_URL = re.compile(r' (src|href)=[\'"]\/')


def is_binary(filepath):
    """Return True if the given filename is binary.
    """
    CHUNKSIZE = 1024
    with io.open(filepath, 'rb') as f:
        while 1:
            chunk = f.read(CHUNKSIZE)
            if '\0' in chunk: # found null byte
                return True
            if len(chunk) < CHUNKSIZE:
                break
    return False


def walk_dir(path, callback, ignore=None):
    ignore = ignore or ()
    for folder, subs, files in os.walk(path):
        ffolder = os.path.relpath(folder, path)
        for filename in files:
            if filename.startswith(ignore):
                continue
            relpath = os.path.join(ffolder, filename) \
                .lstrip('.').lstrip('/')
            callback(relpath)


def make_dirs(*lpath):
    path = os.path.join(*lpath)
    try:
        os.makedirs(os.path.dirname(path))
    except (OSError), e:
        if e.errno != errno.EEXIST:
            raise
    return path


def get_source(filepath):
    source = ''
    with io.open(filepath) as f:
        source = f.read()
    return source


def make_file(filepath, content):
    if not isinstance(content, unicode):
        content = unicode(content, 'utf-8')
    with io.open(filepath, 'w+t') as f:
        f.write(content)


def remove_file(filepath):
    try:
        os.remove(filepath)
    except OSError:
        pass


def absolute_to_relative(content, relpath):
    depth = relpath.count(os.path.sep)
    repl = '../' * depth
    rx_rel_url = r' \1="%s' % repl
    content = re.sub(RX_ASB_URL, rx_rel_url, content)
    return content


def get_processed_regex(processed_files):
    rx_processed = [[
        re.compile(r' (src|href)=[\'"](.*)%s[\'"]' % old),
        r' \1="\2%s"' % new
    ] for old, new in processed_files]
    return rx_processed


def replace_processed_names(content, rx_processed):
    for rxold, rxnew in rx_processed:
        content = re.sub(rxold, rxnew, content)
    return content


def get_file_mdate(filepath):
    mtime = os.path.getmtime(filepath)
    mdate = datetime.utcfromtimestamp(mtime)
    mdate -=  datetime.utcnow() - datetime.now()
    return mdate


def copy_if_has_change(path_in, path_out):
    if os.path.exists(path_out):
        oldt = os.path.getmtime(path_out)
        newt = os.path.getmtime(path_in)
        if oldt == newt:
            return
    shutil.copy2(path_in, path_out)


def _is_protected_type(obj):
    """Determine if the object instance is of a protected type.

    Objects of protected types are preserved as-is when passed to
    to_unicode(strings_only=True).
    """
    return isinstance(obj, (
        types.NoneType,
        int, long,
        datetime.datetime, datetime.date, datetime.time,
        float, Decimal)
    )


def to_unicode(s, encoding='utf-8', strings_only=False, errors='strict'):
    """Returns a unicode object representing 's'. Treats bytestrings using the
    `encoding` codec.

    If strings_only is True, don't convert (some) non-string-like objects.

    --------------------------------
    Copied almost unchanged from Django <https://www.djangoproject.com/>
    Copyright © 2005-2011 Django Software Foundation.
    Used under the modified BSD license.
    """
    # Handle the common case first, saves 30-40% in performance when s
    # is an instance of unicode.
    if isinstance(s, unicode):
        return s
    if strings_only and _is_protected_type(s):
        return s
    encoding = encoding or 'utf-8'
    try:
        if not isinstance(s, basestring):
            if hasattr(s, '__unicode__'):
                s = unicode(s)
            else:
                try:
                    s = unicode(str(s), encoding, errors)
                except UnicodeEncodeError:
                    if not isinstance(s, Exception):
                        raise
                    # If we get to here, the caller has passed in an Exception
                    # subclass populated with non-ASCII data without special
                    # handling to display as a string. We need to handle this
                    # without raising a further exception. We do an
                    # approximation to what the Exception's standard str()
                    # output should be.
                    s = u' '.join([to_unicode(arg, encoding, strings_only,
                        errors) for arg in s])
        elif not isinstance(s, unicode):
            # Note: We use .decode() here, instead of unicode(s, encoding,
            # errors), so that if s is a SafeString, it ends up being a
            # SafeUnicode at the end.
            s = s.decode(encoding, errors)
    except UnicodeDecodeError, e:
        if not isinstance(s, Exception):
            raise UnicodeDecodeError(s, *e.args)
        else:
            # If we get to here, the caller has passed in an Exception
            # subclass populated with non-ASCII bytestring data without a
            # working unicode method. Try to handle this without raising a
            # further exception by individually forcing the exception args
            # to unicode.
            s = u' '.join([to_unicode(arg, encoding, strings_only,
                errors) for arg in s])
    return s


def filter_to_json(source_dict):
    return json.dumps(source_dict)




# -*- coding: utf-8 -*-
"""
# Clay

A rapid prototyping tool

------------
Copyright © 2011 by [Lúcuma labs] (http://lucumalabs.com).  
See `AUTHORS.md` for more details.  
License: [MIT License] (http://www.opensource.org/licenses/mit-license.php).

"""
from .core import *


__version__ = '1.0.9'



# -*- coding: utf-8 -*-
"""
# Clay.core

Main file

"""
from __future__ import absolute_import

import glob
import mimetypes
import os
import socket

from jinja2 import (PackageLoader, ChoiceLoader, FileSystemLoader)
from jinja2.exceptions import TemplateSyntaxError
from shake import (Shake, Settings, Render, Rule, NotFound, send_file,
    Response)

from . import utils, config
from . import p_scss, p_less, p_markdown, p_coffee
from . import pp_pygments, pp_typogrify


class Clay(object):

    def __init__(self, base_dir, settings=None, source_dir=config.SOURCE_DIR):
        if os.path.isfile(base_dir):
            base_dir = os.path.abspath(os.path.dirname(base_dir))
        self.base_dir = base_dir
        self.source_dir = utils.make_dirs(base_dir, source_dir)
        self.build_dir = os.path.join(base_dir, config.BUILD_DIR)

        settings = settings or {}
        self.settings = Settings(config.default_settings, settings,
            case_insensitive=True)
        theme_prefix = self.settings.get('theme_prefix', '').rstrip('/')
        if theme_prefix:
            theme_prefix += '/'
        self.settings['theme_prefix'] = theme_prefix

        self.app = Shake(config.app_settings)
        self._make_render()
        self._enable_pre_processors()
        self._add_urls()

    def _make_render(self):
        loader = ChoiceLoader([
            FileSystemLoader(self.source_dir),
            PackageLoader('clay', config.SOURCE_DIR),
        ])
        self.render = Render(loader=loader)
        self.render.set_filter('json', utils.filter_to_json)

    def _enable_pre_processors(self):
        ext_trans = {}
        processors = self.settings.pre_processors

        for name in processors:
            pr = globals().get('p_' + name)
            if pr and pr.enabled:
                pr.add_extensions(self)
                for ext in pr.extensions_in:
                    ext_trans[ext] = pr.extension_out

        self.ext_trans = ext_trans

    def _add_urls(self):
        self.app.add_urls([
            Rule('/', self.render_view),
            Rule('/<path:path>', self.render_view),
        ])

    def _normalize_path(self, path):
        if '..' in path:
            return self.not_found()
        path = path.strip('/')
        is_dir = os.path.isdir(os.path.join(self.source_dir, path))
        if is_dir:
            path += '/'
        if not path or is_dir:
            path += 'index.html'
        return path

    def _translate_ext(self, old_ext):
        return self.ext_trans.get(old_ext, old_ext)

    def _get_alternative(self, path):
        path = path.strip('/')
        fname, ext = os.path.splitext(path)
        fullpath = os.path.join(config.DEFAULT_TEMPLATES, path)
        if os.path.exists(fullpath):
            return ext, path, fullpath
        
        if path != 'index.html':
            return None, None, None

        pdir = os.path.join(self.source_dir, fname + '.*')
        files = glob.glob(pdir)
        if files:
            fullpath = files[0]
            path = fullpath.replace(self.source_dir, '')
            _, ext = os.path.splitext(path)
            return ext, path, fullpath

        return None, None, None

    def _post_process(self, html):
        html = utils.to_unicode(html)
        processors = self.settings.post_processors

        for name in processors:
            pp = globals().get('pp_' + name)
            if pp and pp.enabled:
                html = pp.process(html)

        return html

    def run(self, host=None, port=None):
        host = host or self.settings.host
        port = port or self.settings.port
        ips = [ip for ip in socket.gethostbyname_ex(socket.gethostname())[2]
            if not ip.startswith("127.")][:1]
        if ips:
            print ' * Your local IP is:', ips[0]
        return self.app.run(host, port)

    def build(self):
        """Generates a static version of the site.
        """
        print '\nGenerating views...\n', '-' * 20
        self.build_views()
        print '\nDone!\n'

    def test_client(self):
        return self.app.test_client()

    def render_view(self, request, path=''):
        """Default controller.
        Render the template at `path` guessing it's mimetype.
        """
        path = self._normalize_path(path)
        fn, ext = os.path.splitext(path)
        real_ext = self._translate_ext(ext)
        fullpath = os.path.join(self.source_dir, path.lstrip('/'))

        if not os.path.exists(fullpath):
            ext, path, fullpath = self._get_alternative(path)
            if not fullpath:
                return self.not_found()

        plain_text_exts = self.settings.plain_text
        if ext in plain_text_exts or utils.is_binary(fullpath):
            return send_file(request, fullpath)

        try:
            resp = self.render(path, **self.settings)
        except TemplateSyntaxError:
            print '-- WARNING:', 'Syntax error while trying to process', \
                    utils.to_unicode(path), 'as a Jinja template.'
            source = utils.get_source(fullpath)
            resp = Response(source)

        if real_ext == '.html':
            resp.data = self._post_process(resp.data)

        resp.mimetype = mimetypes.guess_type('a' + real_ext)[0] or 'text/plain'
        return resp

    def build_views(self):
        processed = []
        views = []

        def callback(relpath_in):
            print relpath_in
            fn, ext = os.path.splitext(relpath_in)
            real_ext = self._translate_ext(ext)
            relpath_out = '%s%s' % (fn, real_ext)

            path_in = os.path.join(self.source_dir, relpath_in)
            path_out = utils.make_dirs(self.build_dir, relpath_out)

            plain_text_exts = self.settings.plain_text
            if ext in plain_text_exts or utils.is_binary(path_in):
                return utils.copy_if_has_change(path_in, path_out)

            try:
                content = self.render.to_string(relpath_in, **self.settings)
            except TemplateSyntaxError:
                print '-- WARNING:', 'Syntax error while trying to process', \
                    utils.to_unicode(relpath_in), 'as a Jinja template.'
                content = utils.get_source(path_in)
            
            if real_ext != ext:
                processed.append([relpath_in, relpath_out])

            if real_ext == '.html':
                content = self._post_process(content)
                return views.append([relpath_in, path_out, content])

            utils.make_file(path_out, content)
            return
        
        utils.walk_dir(self.source_dir, callback, config.IGNORE)
        rx_processed = utils.get_processed_regex(processed)
        views_list = []

        for relpath_in, path_out, content in views:
            content = utils.absolute_to_relative(content, relpath_in)
            content = utils.replace_processed_names(content, rx_processed)
            utils.make_file(path_out, content)
            views_list.append(relpath_in.decode('utf8'))
        
        self.build_views_list(views_list)

    def build_views_list(self, views):
        ignore = self.settings.views_list_ignore
        ignore.append(config.VIEWS_INDEX)
        real_views = []

        for v in views:
            if v in ignore or v.startswith(config.IGNORE):
                continue
            mdate = utils.get_file_mdate(os.path.join(self.source_dir, v))
            fn, ext = os.path.splitext(v)
            real_ext = self._translate_ext(ext)
            v = '%s%s' % (fn, real_ext)
            real_views.append((v, ' / '.join(v.split('/')), mdate))
        
        content = self.render.to_string(config.VIEWS_INDEX, views=real_views)
        final_path = utils.make_dirs(self.build_dir, config.VIEWS_INDEX)
        utils.make_file(final_path, content)

    def not_found(self):
        resp = self.render('notfound.html')
        resp.status_code = 404
        resp.mimetype = 'text/html'
        return resp




# -*- coding: utf-8 -*-
"""
    # Clay.p_scss

    Sassy CSS processor

    http://sass-lang.com/
    https://github.com/Kronuz/pyScss

"""
import os

from jinja2.ext import Extension
try:
    import scss
    enabled = True
except ImportError:
    enabled = False


extensions_in = ('.css.scss', '.scss',)
extension_out = '.css'


def add_extensions(clay):
    css = scss.Scss()

    class SassyCSSExtension(Extension):

        def preprocess(self, source, name, filename=None):
            if name is None or os.path.splitext(name)[1] not in extensions_in:
                return source
            return css.compile(source)
    
    clay.render.add_extension(SassyCSSExtension)




"""
Table of Contents Extension for Python-Markdown
* * *

(c) 2008 [Jack Miller](http://codezen.org)

Dependencies:
* [Markdown 2.1+](http://www.freewisdom.org/projects/python-markdown/)

"""
import markdown
from markdown.util import etree
from markdown.extensions.headerid import slugify, unique, itertext

import re


class TocTreeprocessor(markdown.treeprocessors.Treeprocessor):
    # Iterator wrapper to get parent and child all at once
    def iterparent(self, root):
        for parent in root.getiterator():
            for child in parent:
                yield parent, child

    def run(self, doc):
        div = etree.Element("div")
        last_li = None

        level = 0
        list_stack=[div]
        header_rgx = re.compile("[Hh][123456]")

        # Get a list of id attributes
        used_ids = []
        for c in doc.getiterator():
            if "id" in c.attrib:
                used_ids.append(c.attrib["id"])

        for (p, c) in self.iterparent(doc):
            text = ''.join(itertext(c)).strip()
            if not text:
                continue
                    
            if header_rgx.match(c.tag):
                try:
                    tag_level = int(c.tag[-1])
                    
                    while tag_level < level:
                        list_stack.pop()
                        level -= 1

                    if tag_level > level:
                        newlist = etree.Element("ul")
                        if last_li:
                            last_li.append(newlist)
                        else:
                            list_stack[-1].append(newlist)
                        list_stack.append(newlist)

                        if level == 0:
                            level = tag_level
                        else:
                            level += 1

                    # Do not override pre-existing ids 
                    if not "id" in c.attrib:
                        id = unique(self.config["slugify"](text, '-'), used_ids)
                        c.attrib["id"] = id
                    else:
                        id = c.attrib["id"]

                    # List item link, to be inserted into the toc
                    last_li = etree.Element("li")
                    link = etree.SubElement(last_li, "a")
                    link.text = text
                    link.attrib["href"] = '#' + id

                    if self.config["anchorlink"]:
                        anchor = etree.Element("a")
                        anchor.text = c.text
                        anchor.attrib["href"] = "#" + id
                        anchor.attrib["class"] = "toclink"
                        c.text = ""
                        for elem in c.getchildren():
                            anchor.append(elem)
                            c.remove(elem)
                        c.append(anchor)

                    list_stack[-1].append(last_li)
                except IndexError:
                    # We have bad ordering of headers. Just move on.
                    pass
        
        # searialize and attach to markdown instance.
        ul = div.find('ul')
        if not ul:
            self.markdown.toc = ''
            return
        
        ul.attrib["class"] = "toc"
        toc = self.markdown.serializer(ul)
        for pp in self.markdown.postprocessors.values():
            toc = pp.run(toc)
        self.markdown.toc = toc


class TocExtension(markdown.Extension):

    def __init__(self, **configs):
        self.config = {
            "slugify" : [slugify,
                "Function to generate anchors based on header text-"
                "Defaults to the headerid ext's slugify function."
            ],
            "anchorlink" : [True,
                "True if header should be a self link"
                "Defaults to 0"
            ],
        }
        for key, value in configs:
            self.setConfig(key, value)

    def extendMarkdown(self, md, md_globals):
        tocext = TocTreeprocessor(md)
        tocext.config = self.getConfigs()
        # Headerid ext is set to '>inline'. With this set to '<prettify',
        # it should always come after headerid ext (and honor ids assinged 
        # by the header id extension) if both are used. Same goes for 
        # attr_list extension. This must come last because we don't want
        # to redefine ids after toc is created. But we do want toc prettified.
        md.treeprocessors.add("toc", tocext, "<prettify")

	
def makeExtension(configs={}):
    return TocExtension(configs=configs)




# -*- coding: utf-8 -*-
import shutil

from clay import p_scss
import pytest

from tests.utils import *


def setup_module():
    try:
        shutil.rmtree(clay_.build_dir)
    except OSError:
        pass


def teardown_module():
    try:
        shutil.rmtree(clay_.build_dir)
    except OSError:
        pass


SRC_SCSS = """
@option compress: no;
$main-color: #ce4dd6;
$style: solid;
$side: bottom;
#navbar {
  border-#{$side}: {
    color: $main-color;
    style: $style;
  }
}
""".strip()


EXPECTED_SCSS = """
#navbar {
  border-bottom-color: #ce4dd6;
  border-bottom-style: solid;
}
""".strip()


FILENAME_IN = 'sassy.scss'
FILENAME_OUT = 'sassy.css'


SRC_HTML = """
<link rel="stylesheet" href="foo/bar/%s" />
<p class="scss"></p>""" % FILENAME_IN

EXPECTED_HTML = """
<link rel="stylesheet" href="foo/bar/%s" />
<p class="scss"></p>""" % FILENAME_OUT


def test_scss_enabled():
    assert p_scss.enabled


def test_scss_render():
    filepath = make_view(FILENAME_IN, SRC_SCSS)
    try:
        resp = c.get('/' + FILENAME_IN)
        content = resp.data.strip()
        assert content == EXPECTED_SCSS
    finally:
        remove_file(filepath)


def test_scss_build():
    filepath = make_view(FILENAME_IN, SRC_SCSS)
    filepath_out = get_build_filepath(FILENAME_OUT)
    try:
        clay_.build()
        content = read_file(filepath_out).strip()
        assert content == EXPECTED_SCSS
    finally:
        remove_file(filepath)
        remove_file(filepath_out)


def test_scss_html_replace():
    static_filepath = make_view(FILENAME_IN, SRC_SCSS)
    filepath_out = get_build_filepath(FILENAME_OUT)
    html_filename = 'test_scss.html'
    html_filepath = make_view(html_filename, SRC_HTML)
    html_filepath_out = get_build_filepath(html_filename)
    try:
        clay_.build()
        content = read_file(html_filepath_out)
        assert content == EXPECTED_HTML
    finally:
        remove_file(static_filepath)
        remove_file(filepath_out)
        remove_file(html_filepath)
        remove_file(html_filepath_out)




# -*- coding: utf-8 -*-
"""
# Fenced Code Github-style Extension for Python Markdown

This extension adds Fenced Code Blocks to Python-Markdown.

    >>> import markdown
    >>> text = '''
    ... A paragraph before a fenced code block:
    ...
    ... ````
    ... Fenced code block
    ... ````
    ... '''
    >>> html = markdown.markdown(text, extensions=['fenced_code_gh'])
    >>> print html
    <p>A paragraph before a fenced code block:</p>
    <pre><code>Fenced code block
    </code></pre>

Works with safe_mode also (we check this because we are using the HtmlStash):

    >>> print markdown.markdown(text, extensions=['fenced_code_gh'], safe_mode='replace')
    <p>A paragraph before a fenced code block:</p>
    <pre><code>Fenced code block
    </code></pre>

Include tilde's in a code block and wrap with blank lines:

    >>> text = '''
    ... ````````
    ...
    ... ````
    ... ````````'''
    >>> print markdown.markdown(text, extensions=['fenced_code_gh'])
    <pre><code>
    ````
    </code></pre>

Language tags:

    >>> text = '''
    ... ````python
    ... # Some python code
    ... ````'''
    >>> print markdown.markdown(text, extensions=['fenced_code_gh'])
    <pre><code class="python"># Some python code
    </code></pre>

"""
import re
import markdown


RX_FENCED_BLOCK = re.compile(
    r'''(?P<fence>^`{3,})[ ]*(?P<lang>[^\n]+)?\n(?P<code>.*?)(?P=fence)[ ]*$''',
    re.MULTILINE|re.DOTALL
)
CODE_WRAP = '<pre><code%s>%s</code></pre>'
LANG_TAG = ' class="%s"'


class FencedCodeGhExtension(markdown.Extension):

    def extendMarkdown(self, md, md_globals):
        """ Add FencedBlockPreprocessor to the Markdown instance.
        """
        md.registerExtension(self)

        md.preprocessors.add('fenced_code_block',
            FencedBlockPreprocessor(md),
            "_begin")


class FencedBlockPreprocessor(markdown.preprocessors.Preprocessor):

    def run(self, lines):
        """ Match and store Fenced Code Blocks in the HtmlStash.
        """
        text = "\n".join(lines)
        while 1:
            m = RX_FENCED_BLOCK.search(text)
            if m:
                lang = ''
                if m.group('lang'):
                    lang = LANG_TAG % m.group('lang')

                code = ''.join([
                    '{% raw %}',
                    CODE_WRAP % (lang, self._escape(m.group('code'))),
                    '{% endraw %}'
                ])

                placeholder = self.markdown.htmlStash.store(code, safe=True)
                text = '%s\n%s\n%s'% (text[:m.start()], placeholder, text[m.end():])
            else:
                break
        return text.split("\n")

    def _escape(self, txt):
        """ basic html escaping """
        txt = txt.replace('&', '&amp;')
        txt = txt.replace('<', '&lt;')
        txt = txt.replace('>', '&gt;')
        txt = txt.replace('"', '&quot;')
        return txt


def makeExtension(configs={}):
    return FencedCodeGhExtension(configs=configs)




# -*- coding: utf-8 -*-
import shutil

from clay import p_less
import pytest

from tests.utils import *


def setup_module():
    try:
        shutil.rmtree(clay_.build_dir)
    except OSError:
        pass


def teardown_module():
    try:
        shutil.rmtree(clay_.build_dir)
    except OSError:
        pass


SRC_LESS = """
@the-border: 1px;
@base-color: #111;
@red: #842210;

#header {
  color: @base-color * 3;
  border-left: @the-border;
  border-right: @the-border * 2;
}

#footer {
  color: @base-color + #003300;
  border-color: desaturate(@red, 10%);
}
""".strip()


EXPECTED_LESS = """
#header {
  color: #333333;
  border-left: 1px;
  border-right: 2px;
}
#footer {
  color: #114411;
  border-color: #7d2717;
}
""".strip()


FILENAME_IN = 'less.less'
FILENAME_OUT = 'less.css'


SRC_HTML = """
<link rel="stylesheet" href="foo/bar/%s" />
<p class="scss"></p>""" % FILENAME_IN


EXPECTED_HTML = """
<link rel="stylesheet" href="foo/bar/%s" />
<p class="scss"></p>""" % FILENAME_OUT


def test_less_enabled():
    assert p_less.enabled


def test_less_render():
    filepath = make_view(FILENAME_IN, SRC_LESS)
    try:
        resp = c.get('/' + FILENAME_IN)
        content = resp.data.strip()
        assert content == EXPECTED_LESS
    finally:
        remove_file(filepath)


def test_less_make():
    filepath = make_view(FILENAME_IN, SRC_LESS)
    filepath_out = get_build_filepath(FILENAME_OUT)
    try:
        clay_.build()
        content = read_file(filepath_out).strip()
        assert content.strip() == EXPECTED_LESS
    finally:
        remove_file(filepath)
        remove_file(filepath_out)


def test_less_html_replace():
    static_filepath = make_view(FILENAME_IN, SRC_LESS)
    filepath_out = get_build_filepath(FILENAME_OUT)
    html_filename = 'test_less.html'
    html_filepath = make_view(html_filename, SRC_HTML)
    html_filepath_out = get_build_filepath(html_filename)
    try:
        clay_.build()
        content = read_file(html_filepath_out)
        assert content == EXPECTED_HTML
    finally:
        remove_file(static_filepath)
        remove_file(filepath_out)
        remove_file(html_filepath)
        remove_file(html_filepath_out)




# -*- coding: utf-8 -*-
import io
import os
import shutil

from clay import Clay
import pytest

from .utils import *


def setup_module():
    try:
        shutil.rmtree(clay_.build_dir)
    except OSError:
        pass


def teardown_module():
    try:
        shutil.rmtree(clay_.build_dir)
    except OSError:
        pass


def test_common():
    resp = c.get('/index.html')
    assert resp.status_code == HTTP_OK
    assert resp.mimetype == 'text/html'
    assert len(resp.data) > 100


def test_view_not_found():
    resp = c.get('/qwertyuiop.bar')
    assert '<!-- not found -->' in resp.data
    assert resp.status_code == HTTP_NOT_FOUND
    assert resp.mimetype == 'text/html'


def test_render_non_ascii_filenames():
    resp = c.get(u'/mañana.txt')
    assert resp.status_code == HTTP_OK
    assert resp.mimetype == 'text/plain'


def test_render_non_utf8_content():
    resp = c.get('/iso-8859-1.txt')
    assert resp.status_code == HTTP_OK
    assert resp.mimetype == 'text/plain'


def test_build():
    try:
        shutil.rmtree(clay_.build_dir)
    except OSError:
        pass
    clay_.build()

    assert os.path.isdir(clay_.build_dir)
    path = os.path.join(clay_.build_dir, 'index.html')
    assert os.path.isfile(path)


def test_build_overwrite():
    filename = os.path.join(clay_.build_dir, 'index.html')
    bad_data = u':('
    make_file(filename, bad_data)
    clay_.build()
    
    new_data = read_file(filename)
    assert new_data != bad_data


def test_build_absolute2relative():
    filename1 = os.path.join(clay_.build_dir, 'index.html')
    filename2 = os.path.join(clay_.build_dir, 'foo', 'index.html')
    clay_.build()

    c1 = read_file(filename1)
    assert '<link href="styles/test.css"' in c1
    c2 = read_file(filename2)
    assert '<link href="../styles/test.css"' in c2




# -*- coding: utf-8 -*-
import shutil

from clay import p_markdown
import pytest

from tests.utils import *


def setup_module():
    try:
        shutil.rmtree(clay_.build_dir)
    except OSError:
        pass


def teardown_module():
    try:
        shutil.rmtree(clay_.build_dir)
    except OSError:
        pass


# SRC_SCSS = """
# @option compress: no;
# $main-color: #ce4dd6;
# $style: solid;
# $side: bottom;
# #navbar {
#   border-#{$side}: {
#     color: $main-color;
#     style: $style;
#   }
# }
# """.strip()


# EXPECTED_SCSS = """
# #navbar {
#   border-bottom-color: #ce4dd6;
#   border-bottom-style: solid;
# }
# """.strip()


FILENAME_IN = 'test.md'
FILENAME_OUT = 'test.html'


# SRC_HTML = """
# <link rel="stylesheet" href="foo/bar/%s" />
# <p class="scss"></p>""" % FILENAME_IN

# EXPECTED_HTML = """
# <link rel="stylesheet" href="foo/bar/%s" />
# <p class="scss"></p>""" % FILENAME_OUT


def test_markdown_enabled():
    assert p_markdown.enabled


# def test_scss_render():
#     filepath = make_view(FILENAME_IN, SRC_SCSS)
#     try:
#         resp = c.get('/' + FILENAME_IN)
#         content = resp.data.strip()
#         assert content == EXPECTED_SCSS
#     finally:
#         remove_file(filepath)


# def test_scss_build():
#     filepath = make_view(FILENAME_IN, SRC_SCSS)
#     filepath_out = get_build_filepath(FILENAME_OUT)
#     try:
#         clay_.build()
#         content = read_file(filepath_out).strip()
#         assert content == EXPECTED_SCSS
#     finally:
#         remove_file(filepath)
#         remove_file(filepath_out)


# def test_scss_html_replace():
#     static_filepath = make_view(FILENAME_IN, SRC_SCSS)
#     filepath_out = get_build_filepath(FILENAME_OUT)
#     html_filename = 'test_scss.html'
#     html_filepath = make_view(html_filename, SRC_HTML)
#     html_filepath_out = get_build_filepath(html_filename)
#     try:
#         clay_.build()
#         content = read_file(html_filepath_out)
#         assert content == EXPECTED_HTML
#     finally:
#         remove_file(static_filepath)
#         remove_file(filepath_out)
#         remove_file(html_filepath)
#         remove_file(html_filepath_out)




# -*- coding: utf-8 -*-
import io
import os

from clay.manage import new, build, run, get_settings, get_current
import pytest


def get_cwd():
    return os.path.dirname(os.path.abspath(__file__)) or '.'


def test_get_settings():
    cwd = get_cwd()
    settings = get_settings(cwd, filename='_test.yml')
    assert settings


def test_get_current():
    os.chdir(os.path.dirname(__file__))
    expected = os.getcwd()
    clay_ = get_current()
    assert clay_.base_dir == expected




# -*- coding: utf-8 -*-
import io
import os
import re
from setuptools import setup


PACKAGE = 'clay'
THIS_DIR = os.path.dirname(__file__).rstrip('/')


def get_path(*args):
    return os.path.join(THIS_DIR, *args)


def read_from(filepath):
    with io.open(filepath) as f:
        source = f.read()
    return source


def get_version():
    data = read_from(get_path(PACKAGE, '__init__.py'))
    version = re.search(r"__version__\s*=\s*'([^']+)'", data).group(1)
    return version.encode('utf-8')


def find_package_data(root, include_files=None):
    include_files = include_files or ['.gitignore',]
    files = []
    src_root = get_path(root).rstrip('/') + '/'
    for dirpath, subdirs, filenames in os.walk(src_root):
        path, dirname = os.path.split(dirpath)
        if dirname.startswith(('.', '_')):
            continue
        dirpath = dirpath.replace(src_root, '')
        for filename in filenames:
            is_valid_filename = not (
                filename.startswith('.') or
                filename.endswith('.pyc')
            )
            include_it_anyway = filename in include_files

            if is_valid_filename or include_it_anyway:
                files.append(os.path.join(dirpath, filename))
    return files


def find_packages_data(*roots):
    return dict([(root, find_package_data(root)) for root in roots])


def get_requirements():
    data = read_from(get_path('requirements.txt'))
    lines = map(lambda s: s.strip(), data.splitlines())
    return [l for l in lines if l and not l.startswith('#')]


def run_tests():
    import sys, subprocess
    errno = subprocess.call([sys.executable, 'runtests.py'])
    raise SystemExit(errno)


setup(
    name = 'Clay',
    version = get_version(),
    author = 'Juan-Pablo Scaletti',
    author_email = 'juanpablo@lucumalabs.com',
    packages = [PACKAGE],
    package_data = find_packages_data(PACKAGE, 'tests'),
    zip_safe = False,
    url = 'http://github.com/lucuma/Clay',
    license = 'MIT license (http://www.opensource.org/licenses/mit-license.php)',
    description = 'A rapid prototyping tool',
    long_description = read_from(get_path('README.rst')),
    install_requires = get_requirements(),
    classifiers = [
        'Development Status :: 4 - Beta',
        'Environment :: Web Environment',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Topic :: Internet :: WWW/HTTP :: Dynamic Content',
        'Topic :: Software Development :: Libraries :: Python Modules'
    ],
    test_suite = '__main__.run_tests',
    entry_points = {'console_scripts': ['clay = clay.manage:main']},
)



# -*- coding: utf-8 -*-
"""
    # Clay.p_coffee

    http://jashkenas.github.com/coffee-script/

"""
import os
import subprocess

from jinja2.ext import Extension

from .utils import get_source, remove_file


COMMAND = 'coffee'

try:
    subprocess.check_output([COMMAND, '--version'])
    enabled = True
except Exception:
    enabled = False

extensions_in = ('.js.coffee',  '.coffee',)
extension_out = '.js'


def add_extensions(clay):

    class CoffeeExtension(Extension):

        def preprocess(self, source, name, filename=None):
            if name is None or os.path.splitext(name)[1] not in extensions_in:
                return source
            
            path = os.path.join(clay.source_dir, name)
            fn, ext = os.path.splitext(path)
            filepath_out = fn + extension_out
            try:
                args = [COMMAND, '--compile', path]
                subprocess.call(args)
                content = get_source(filepath_out)
            except subprocess.CalledProcessError:
                content = ''
            finally:
                remove_file(filepath_out)
            return content

    clay.render.add_extension(CoffeeExtension)




# -*- coding: utf-8 -*-
"""Superscipt extension for Markdown.

To superscript something, place a carat symbol, '^', before and after the
text that you would like in superscript:  6.02 x 10^23^
The '23' in this example will be superscripted.  See below.

Examples:

>>> import markdown
>>> md = markdown.Markdown(extensions=['superscript'])
>>> md.convert('This is a reference to a footnote^1^.')
u'<p>This is a reference to a footnote<sup>1</sup>.</p>'

>>> md.convert('This is scientific notation: 6.02 x 10^23^')
u'<p>This is scientific notation: 6.02 x 10<sup>23</sup></p>'

>>> md.convert('This is scientific notation: 6.02 x 10^23. Note lack of second carat.')
u'<p>This is scientific notation: 6.02 x 10^23. Note lack of second carat.</p>'

>>> md.convert('Scientific notation: 6.02 x 10^23. Add carat at end of sentence.^')
u'<p>Scientific notation: 6.02 x 10<sup>23. Add a carat at the end of sentence.</sup>.</p>'

Paragraph breaks will nullify superscripts across paragraphs. Line breaks
within paragraphs will not.

"""

import markdown


SUPERSCRIPT_RE = r'(\^)([^\^]*)\2'  # the number is a superscript^2^


class SuperscriptPattern(markdown.inlinepatterns.Pattern):
    """ Return a superscript Element (`word^2^`). """

    def handleMatch(self, m):
        supr = m.group(3)
        
        text = supr
        
        el = markdown.etree.Element("sup")
        el.text = markdown.AtomicString(text)
        return el


class SuperscriptExtension(markdown.Extension):
    """ Superscript Extension for Python-Markdown. """

    def extendMarkdown(self, md, md_globals):
        """ Replace superscript with SuperscriptPattern """
        md.inlinePatterns['superscript'] = SuperscriptPattern(SUPERSCRIPT_RE, md)


def makeExtension(configs={}):
    return SuperscriptExtension(configs=configs)

